
#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
using System.Xml;

#endregion

namespace CheeseBreakout
{
    /// <summary>
    /// This is a game component that implements IUpdateable.
    /// </summary>
    public partial class StatusDisplay : Microsoft.Xna.Framework.DrawableGameComponent
    {

        XNAFont textWriter;
        IGetStatusText statusSource;


        public StatusDisplay(Game game, IGetStatusText inStatusSource)
            : base(game)
        {
            statusSource = inStatusSource;
        }


        /// <summary>
        /// Allows the game component to perform any initialization it needs to before starting
        /// to run.  This is where it can query for any required services and load content.
        /// </summary>
        public override void Initialize()
        {
            this.DrawOrder = 2;

            base.Initialize();
        }


        protected override void LoadGraphicsContent(bool loadAllContent)
        {
            if (loadAllContent)
            {
                textWriter = XNAFont.LoadFont(this.GraphicsDevice,
                                "font/scorefont.dds.xml", "font/scorefont.dds");
            }
            base.LoadGraphicsContent(loadAllContent);
        }

        public override void Draw(GameTime gameTime)
        {
            string text = statusSource.GetStatus();

            textWriter.OutputText(
                text, // text to write
                (int)((Game.Window.ClientBounds.Width - textWriter.TextLength(text)) / 2.0f),
                0,
                Color.Blue);
            base.Draw(gameTime);
        }

        /// <summary>
        /// Allows the game component to update itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Update(GameTime gameTime)
        {
            // TODO: Add your update code here

            base.Update(gameTime);
        }
    }

    public class XNAFont : IDisposable
    {
        string strFontFamily;
        Texture2D texture;
        SpriteBatch fontRenderer;
        public int Height;

        struct XNAFontUV
        {
            public float u, v;
            public int Width;

            public XNAFontUV(float u, float v, int Width)
            {
                this.u = u;
                this.v = v;
                this.Width = Width;
            }
        }

        Dictionary<char, XNAFontUV> UVData = new Dictionary<char, XNAFontUV>();

        public void OutputText(string str, int x, int y, Color col)
        {
            int iCurrentX = x;
            int iCurrentY = y;

            fontRenderer.Begin(SpriteBlendMode.AlphaBlend);
            foreach (char c in str)
            {
                XNAFontUV uv = UVData[c];

                fontRenderer.Draw(texture,
                    new Rectangle(iCurrentX, iCurrentY,
                        uv.Width, Height),
                    new Rectangle((int)(uv.v * texture.Height),
                                  (int)((1.0f - uv.u) * texture.Width),
                                  uv.Width, Height),
                    col);

                iCurrentX += uv.Width;
            }
            fontRenderer.End();
        }

        public int TextLength(string str)
        {
            int length = 0;
            foreach (char c in str)
            {
                length += UVData[c].Width;
            }
            return length;
        }

        public static XNAFont LoadFont(GraphicsDevice device, string strXML, string strDDS)
        {
            XNAFont ret = new XNAFont();

            System.Xml.XmlTextReader tr = new XmlTextReader(strXML);
            while (tr.Read())
            {
                if (tr.Name == "Font")
                {
                    if (tr.AttributeCount > 0)
                    {
                        ret.strFontFamily = tr.GetAttribute("Family");
                        ret.Height = Convert.ToInt32(tr.GetAttribute("Height"));
                    }
                }
                else if (tr.Name == "Char")
                {
                    string str = tr.GetAttribute("c");
                    str = str.Replace("&qwo;", "\"");
                    str = str.Replace("&amp;", "&");
                    str = str.Replace("&lt;", "<");
                    str = str.Replace("&gt;", ">");
                    char c = str[0];

                    ret.UVData.Add(c,
                        new XNAFontUV(
                            Convert.ToSingle(tr.GetAttribute("u")),
                            Convert.ToSingle(tr.GetAttribute("v")),
                            Convert.ToInt32(tr.GetAttribute("width"))));

                }
            }
            tr.Close();

            ret.texture = Texture2D.FromFile(device, strDDS);
            ret.fontRenderer = new SpriteBatch(device);

            return ret;
        }

        #region IDisposable Members

        public void Dispose()
        {
            if (texture != null)
                texture.Dispose();

            if (fontRenderer != null)
                fontRenderer.Dispose();

            texture = null;
            fontRenderer = null;
        }

        #endregion
    }
}


